#    Acquisition for AGATA using Quixtream interface

set frame  Quix
set title "Quixtream Acquisition"
set icon   Quix

set started 0

set Config  "c:/MIDAS/AGATA/QuixDA/bin/Win32/daq.xml"
set File1   "d:/AGATAData/OutputData1.dat"
set File2   "d:/AGATAData/OutputData2.dat"
set Data1   "d:/AGATAData/OutputData1.dat"
set Hist1   "d:/AGATAData/HistogramData1.msf"
set Info    "Agata_Raw_Data"

set Offset 0
set Length 65536
set OverWrite 1
set AcquireNumber 1
set Acquiring 0
set AcqDelay 5000

set execute ""
set response ""

set reportflag 1
set abortflag 0

set OffsetMenu [list 0 50K 100K 150K 200K 500K 1M 2M 5M 10M 20M 30M 40M 50M 60M]
set Offsets [list 0 50000 100000 150000 200000 500000 1000000 2000000 5000000 10000000 20000000 30000000 40000000 50000000 60000000]
set LengthMenu [list 2K 5K 10K 20K 50K 64K]
set Lengths [list 2000 5000 10000 20000 50000 65536]




proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :toplevel T
         wm protocol $frame WM_DELETE_WINDOW "destroy $frame"

    :galley G :deaf T :strings $text :font fixed
    :show T
}

proc make-frame {} {
   global env Config File1 File2 Data1 Hist1 Offset Length OverWrite
   global OffsetMenu LengthMenu AcquireNumber

  :frame [self frame-name] :label "[self frame-title]"  :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 0; :row-gap 10


  :panel P1 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :message M10 :label "Acquisition Options" :bold T
    :next-row T
    :text CONFIG   :label "Quixstream Configuration file" :shifters T :w 40 :valuex 24c :value $Config
    :button BROWSE1 :label "Browse"
    :next-row T
    :text FILE1    :label "Output file for channel 1"     :shifters T :w 40 :valuex 24c :value $File1
    :button BROWSE2 :label "Browse"
    :button AUTO1   :label "Auto"
    :next-row T
    :text FILE2    :label "Output file for channel 2"     :shifters T :w 40 :valuex 24c :value $File2
    :button BROWSE3 :label "Browse"
    :button AUTO2   :label "Auto"
    :next-row T

    :button ACQUIRE :label "Acquire Single Sample"
    :next-row T

    :number NUMBER  :label "Number of Samples to Acquire" :min 1 :max 99 :w 2 :value $AcquireNumber
    :button BULK    :label "Acquire Samples"
    :button ABORT   :label "Abandon Acquire"

  :panel P2 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :message M20 :label "Viewer Options" :bold T
    :next-row T
    :text SOURCE   :label "Data File" :shifters T :w 40 :valuex 24c :value $Data1
    :button BROWSE4 :label "Browse"
    :next-row T
    :text SPEX    :label "Destination Histogram"    :shifters T  :w 40 :valuex 24c :value $Hist1
    :button BROWSE5 :label "Browse"
    :next-row T 
    :text OFFSET :label "Start Offset" :w 10 :value $Offset 
    :menubutton OFFSETM  :label ""  :strings $OffsetMenu 
    :text LENGTH  :label "Sample Length"   :w 10  :value $Length
    :menubutton LENGTHM  :label ""  :strings $LengthMenu
    :checkbox OVERWRITE :label "Overwrite Destination?" :value $OverWrite
    :next-row T

    :button SHOW :label "Show Sample"
    :button MAKE :label "Extract Sample as Histogram"
    :button DISP :label "Extract Sample as Histogram and Display"

   open-frame

   :show T
}


proc start args {
   global started 

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   
}


proc click arg {
   global File1 File2 OverWrite
   global Offsets Lengths AcquireNumber Acquiring

#insert-debug "$arg"

   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer
   switch -glob $w  {
 
      BROWSE*   {click-browse [string range $w 6 end]}
      OVERWRITE {set OverWrite $v}
      NUMBER    {set AcquireNumber $v}
      ABORT     {set Acquiring 0}
      OFFSETM   {click-offset [lindex $Offsets $i]}
      LENGTHM   {click-length [lindex $Lengths $i]}

      AUTO1    {click-file1 [file join [file dirname $File1] Ch1_[date +%Y%h%d-%H.%M.%S].dat]}
      AUTO2    {click-file2 [file join [file dirname $File2] Ch2_[date +%Y%h%d-%H.%M.%S].dat]}
   
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}


proc click-offset {vl} {
    global Offset

    set Offset $vl
    :modify OFFSET :value $Offset
}

proc click-length {vl} {
    global Length

    set Length $vl
    :modify LENGTH :value $Length
}

proc click-spex {vl} {
    global Hist1

    set Hist1 $vl
}

proc click-source {vl} {
    global Data1

    set Data1 $vl
}

proc click-config {vl} {
    global Config

    set Config $vl
}

proc click-file1 {vl} {
    global File1

    set File1 $vl
    :modify FILE1 :value $File1
}

proc click-file2 {vl} {
    global File2

    set File2 $vl
    :modify FILE2 :value $File2
}

proc click-acquire {w} {
    global env
    global Config File1 File2
    global execute response abortflag

     set-footer "acquiring data ........"
     set-busy

      set execute "exec [file join $env(MIDASBASE) AGATA QuixDA bin Win32 daq] /o61 /c$Config /1$File1 /2$File2"
      set z [catch {eval $execute} m]
      set response "$m"
      if {$z != 0} then {
         abort_report
      } else {
         progress_report 1
      }

     clear-footer
     clear-busy

    return $z

}

proc click-bulk {w} {
    global AcquireNumber Acquiring  File1 File2 AcqDelay

    set DT [date +%Y%h%d-%H.%M.%S]

    if {[midas-confirmation "You are about to acquire $AcquireNumber samples\nAre you sure you wish to continue?"] == 0} {return}
    
    set Acquiring 1

    for {set i 0} {$i < $AcquireNumber} {incr i} {
        if {$Acquiring == 0} {set-footer "Data Acquire Abandoned"; return}

        if {$i > 0} {after $AcqDelay}

        click-file1 [file join [file dirname $File1] Ch1_[set DT]#[set i].dat]
        click-file2 [file join [file dirname $File2] Ch2_[set DT]#[set i].dat]

        set z [click-acquire BULK]
        if {$z != 0} {set Acquiring 0}
    }

    set-footer "Data Acquire Complete"
}

proc abort_report {} {
   global execute response abortflag

   set-footer "program terminated abnormally"
   set message "program terminated abnormally\n\
      \nexecuted: \"$execute\"\
      \nresponse received: \"$response\"\
      "
   if {[string length $message] < 1000} then {
      set abortflag [midas-confirmation "$message" "continue" "abort"]
   } else {
      midas-information "$message"
      set abortflag [midas-confirmation "report has been placed in separate window because of its size" "continue" "abort"]
   }
}

proc progress_report {w} {
   global execute response reportflag

   if {$reportflag == 0} then return

   if {$w == 0} then {
      midas-report "executed: \"$execute\"\
       \nresponse received: \n\"$response\"\
       "
   } else {
      midas-information "executed: $execute\nresponse received: \n$response"
   }
}


proc click-show {args} {
    global env
    global Length Offset Data1 Hist1 Info
    global execute response abortflag

      set execute "exec [file join $env(MIDASBASE) AGATA QuixDA bin Win32 show] /o$Offset /l$Length /f$Data1"
      set z [catch {eval $execute} m]
      set response "$m"
      if {$z != 0} then {
         abort_report
      } else {
          progress_report 1
      }
}

proc click-make {args} {
    global env
    global Length Offset Data1 Hist1 Info OverWrite 
    global execute response abortflag

      if {[file exists $Hist1] && $OverWrite == 0} {midas-warning "File $Hist1 exists and you do not want to overwrite it."; return}

      catch {inform sd unshow; file delete $Hist1}

      set execute "exec [file join $env(MIDASBASE) AGATA QuixDA bin Win32 show] /o$Offset /l$Length /f$Data1 /s$Hist1 /i$Info\($Offset\)"
      set z [catch {eval $execute} m]
      set response "$m"
      if {$z != 0} then {
         abort_report
      } else {
         set-footer "$Hist1 created"
      }
}

proc click-disp {args} {
    global Hist1
    global response

    click-make
    send sd show $Hist1
    set overflows [first [split $response \n]]
    set syncs [second [split $response \n]]
    inform sd unset fittags
    inform sd lappend fittags
    if {[string compare -length 13 "overflow at: " $overflows] == 0} {
        foreach item [string range $overflows 13 end] {inform sd lappend fittags $item}
    }
    inform sd restore-tags
    if {[string compare -length 9 "sync at: " $syncs] == 0} {
        foreach item [string range $syncs 9 end] {inform sd make-pointer $item}
    }
}

proc click-browse {w} {
    global env
    global Config File1 File2 Data1 Hist1 

     switch $w {
         1  {
           set directory [file dirname $Config]
	   set types {
	      { {XML Files} {*.xml} }
	      { {All Files } {*} }
	   }
           set mode Open
         }
         2  {
           set directory [file dirname $File1]
	   set types {
	      { {Waveform Files} {*.dat} }
	      { {All Files } {*} }
	   }
           set mode Save
         }
         3  {
           set directory [file dirname $File2]
	   set types {
	      { {Waveform Files} {*.dat} }
	      { {All Files } {*} }
	   }
           set mode Save
         }
         4  {
           set directory [file dirname $Data1]
	   set types {
	      { {Waveform Files} {*.dat} }
	      { {All Files } {*} }
	   }
           set mode Open
         }
         5  {
           set directory [file dirname $Hist1]
	   set types {
	      { {Histogram Files} {*.msf} }
	      { {All Files } {*} }
	   }
           set mode Save
         }
     }

    	option add *Canvas.background  #c0c020

	set file [tk_get[set mode]File -parent [self frame-name] \
		  -title "File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$file == ""} then {return}

     switch $w {
         1  {set Config $file; :modify CONFIG :value $Config}
         2  {set File1 $file; :modify FILE1 :value $File1}
         3  {set File2 $file; :modify FILE2 :value $File2}
         4  {set Data1 $file; :modify SOURCE :value $Data1}
         5  {set Hist1 $file; :modify SPEX :value $Hist1}
     }

}
