# Core_TX_RX.tcl.tcl
set rx_error_last 0 

proc run_test {} {
global %%Waiting
set dg_status [click-status]
InsertLog "Using Core RX to test each optical transmitter "

if { [SelectFpga "Core: Virtex for core ADCs"] == -1 } then { 
	WaitForUser "No Core card available !" 
	return
}
# set the basics - inpsection and main enables
	setup_offset
# enable the laser
	Set_Register 0 0x75
	Set_Register 4 7
	Set_Register 7 0x300

# check the status register for DCM locked.
 set val [Get_Register 1 ]
 if { $val != 0xF201 } {
	InsertLog "Core status  fault : ([format "0x%x" $val])"
	WaitForUser "Continue  ?"
	set rtnd [ set %%Waiting ]
	if { $rtnd == -2 } {
		return
	}
}
set Board "Core: Virtex for Segment ADC card 2"
	foreach channel { 1 2 3 4 5 6  } { 
		WaitForUser "Now channel $channel .... OK?"
		set rtnd [ set %%Waiting ]
		if { $rtnd == -2 } {
			break
		}
	# set up the transmitter to send the sync
		setup_Tx_sync $Board  $channel 
	# check at the core that it's coming in ok
		check_sync
	# transmit data ramp
		setup_Tx_ramp $Board $channel
	# check at the rx that the ramp is stable and not giving error counts
		check_ramp
	# insert a single error and check it is seen by the rx
		setup_Tx_insert $Board $channel
		}
	
	WaitForUser "Test completed .... OK?"
	set rtnd [ set %%Waiting ]
	if { $rtnd == -2 } {
		break
	}

	if { $rtnd == -2 } {
		InsertLog "Ended "
		return
	}
InsertLog "Script completed "
}

proc check_insert { }  {
global rx_error_last
	if { [SelectFpga "Core: Virtex for core ADCs"  ] == -1 } then { 
		WaitForUser "No Core: Virtex for core ADCs  available !" 
		return
	}
	set err2 [ Get_Register 0x33 ]
	if { [ expr $rx_error_last + 1 ]  != $err2 } { 
		InsertLog "Insert error failed : 1: $rx_error_last  and 2: $err2 "
	}
}

proc check_ramp { }  {
global rx_error_last
	if { [SelectFpga "Core: Virtex for core ADCs"  ] == -1 } then { 
		WaitForUser "No Core: Virtex for core ADCs  available !" 
		return
	}
	set stat [ Get_Register 0x30 ]
	if { [ expr $stat & 0xFFF ] != 0  } { 
		InsertLog "Receiving Test ramp - status error : ([format "0x%x" $stat]) "
	}
	Set_Register 0x32 1
	set err1 [ Get_Register 0x33 ]
	set rx_error_last [ Get_Register 0x33 ]
	if { $err1 != $rx_error_last } { 
		InsertLog "Error count changing : 1: $err1  and 2: $rx_error_last "
	}
}

proc check_sync { } {
	if { [SelectFpga "Core: Virtex for core ADCs"  ] == -1 } then { 
		WaitForUser "No Core: Virtex for core ADCs  available !" 
		return
	}
# set the receiver into alignment mode and check the status = 0xda01 and data is FCOO	
	Set_Register 0x32 0
	set stat [ Get_Register 0x30 ]
	if { $stat != 0xDA01 } {
		InsertLog "Error : Receiving sync pattern - status : ([format "0x%x" $stat]) "
	}
	set rx_data [ Get_Register 0x31]
	if { $rx_data != 0xFC00 } {
		InsertLog "Error : Receiving sync pattern - data : ([format "0x%x" $rx_data]) "
	}
}

proc setup_Tx_sync { Board Channel } {
	if { [SelectFpga $Board ] == -1 } then { 
		WaitForUser "No $Board available !" 
		return
	}
	Set_Register [expr ( $Channel << 4 ) + 3 ] 0x7c00
	Set_Register [expr ( $Channel << 4 ) + 4 ] 0
	InsertLog "$Board: $Channel =>Transmitting sync pattern"
}

proc setup_Tx_ramp { Board Channel } {
	if { [SelectFpga $Board ] == -1 } then { 
		WaitForUser "No $Board available !" 
		return
	}
	Set_Register [expr ( $Channel << 4 ) + 3 ] 0
	Set_Register [expr ( $Channel << 4 ) + 4 ] 1
	InsertLog "$Board: $Channel =>Transmitting ramp"
}
proc setup_Tx_insert { Board Channel } {
	if { [SelectFpga $Board ] == -1 } then { 
		WaitForUser "No $Board available !" 
		return
	}
	Set_Register [expr ( $Channel << 4 ) + 5 ] 1
	Set_Register [expr ( $Channel << 4 ) + 5 ] 0
}

run_test
# InsertLog "register at offset 2 has value [Get_Register 2]"
# potential Fpgas are 
#       "Core: Virtex for Segment ADC card 1" 
#       "Core: Virtex for Segment ADC card 2" 
#       "Core: Virtex for core ADCs" 
#       "Core: Main Board" 
#       "Segment: Virtex for Segment ADC card 1" 
#       "Segment: Virtex for Segment ADC card 2" 
#       "Segment: Virtex for Segment ADC card 3" 
#       "Segment: Virtex for Segment ADC card 4" 
#       "Segment: Main Board" 

#    syntax  Set_Register address_offset  value
#    syntax  Get_Register address_offset
#    returns value
# WaitForUser "Data synchronised ?"

# InsertLog "wait returned [set %%Waiting]"

#values of %%Waiting are  -1 (continue)   and   -2  (Abort)
