#Prom_load.tcl
set Prom_control 0x880
set Prom_status 0x881
set Prom_base 0x800
set Prom_limit 128
# Just this simple set of data for a prototype core
set Prom_contents { C 1 0 0 0 P 0 0 }
#' set CurrentScript "$env(HOME)"
set CurrentFile "W:/AGATA_Digitiser_Database"
set Prom_extra {}
set BoardType "U"
set BoardSerial "1"
set BoardRevision "A"
set BoardMod "0"

proc run_test {} {
	global Prom_control Prom_status Prom_base Prom_limit Prom_contents Prom_extra
	global %%Waiting CurrentFile
	global BoardType BoardSerial BoardRevision BoardMod 

	if { [SelectFpga "Core: Virtex for core ADCs"] == -1 } then { 
		WaitForUser "No Core card available !" 
		return
	}
# check to see if it has read in correctly 
	set a 0
	while { $a != 4 } {
		set a [ Get_Register $Prom_status ]
		if { $a != 4 } {
			InsertLog "Prom has read back unsuccessfully. Status : $a "
			WaitForUser "Continue to Retry or Abort to end?"
			set rtnd [ set %%Waiting ]
			if { $rtnd == -2 } {
				return
			}
		Set_Register 0 0x1000		
		Set_Register 0 0		
		} 
	}
# check to see if it's already been written
	set a [ Get_Register $Prom_base ]
	if { $a == "C" || $a == "S" } {
		InsertLog "Prom has already been written "
		WaitForUser "Continue to Overwrite or Abort to end?"
		set rtnd [ set %%Waiting ]
		if { $rtnd == -2 } {
			return
		}
	}

 	InsertLog "Formatting Prom "

 # write the whole prom with spaces to start with
	for { set i 0 } { $i < 128 } { incr i } {
		InsertLog "Writing address $i "
		set a [ Prom_write $i 0x20 ]
		if { $a != 0 } {
			return
		}
	}
	InsertLog "Prom filled with spaces "

	SelectFile
	Init_contents
	set line ""

	set FileId [ open $CurrentFile r ]
	while { [ gets $FileId line ] >= 0 } {
		set firstchar [ string index $line 0 ]
		switch $firstchar {
			+ { continue }
			T { set BoardType [ ExtractChar $line ] }
			S { set BoardSerial [ ExtractString $line ] }
			R { set BoardRevision [ ExtractChar $line ] }
			M { set BoardMod [ ExtractString $line ] }
			( 0 - 9 ) { ExtractAddressed_Value $line }
			default { continue }
		}
	}
	InsertLog "Type 		: $BoardType"
	InsertLog "Serial 	: $BoardSerial"
	InsertLog "Revision 	: $BoardRevision"
	InsertLog "Mod state 	: $BoardMod"
# put the data into the format for the prom writing process
	
	Prom_build 
	InsertLog "Contents : $Prom_contents"
	
	WaitForUser "Continue ? "
	set rtnd [ set %%Waiting ]
	if { $rtnd == -2 } {
		return $rtnd
	}

	InsertLog "Writing board data"
	set i 0 
	foreach val $Prom_contents  {
		InsertLog "Writing address $i "
		set a [ Prom_write $i [ scan "$val" %c ] ]
		if { $a != 0 } {
			return
		}
		incr i
	}

	InsertLog "Script Complete ....."

}
proc Init_contents	{} {
	global Prom_limit Prom_contents 

	set Prom_contents { }
	for { set i 0 } { $i < $Prom_limit } { incr i } { 
		lappend Prom_contents 0
	}
}

proc Prom_build {} {
	global Prom_limit Prom_contents Prom_extra
	global BoardType BoardSerial BoardRevision BoardMod 
	
	set Prom_contents [ lreplace $Prom_contents 0 0 $BoardType ]
	set Prom_contents [ lreplace $Prom_contents 5 5 $BoardRevision ]
	set dig4it [ Digit4List $BoardSerial ]
	set i 1
	foreach val $dig4it {
		set Prom_contents [ lreplace $Prom_contents $i $i $val ]
		incr i
	}
	set dig2it [ Digit2List $BoardMod ]
	set i 6
	foreach val $dig2it {
		set Prom_contents [ lreplace $Prom_contents $i $i $val ]
		incr i
	}
}

proc Digit4List { val } {
	set digit { 0 0 0 0 }
	set a [expr $val / 1000 ]
	set digit [ lreplace $digit 	3 3 $a ]
	set val [ expr $val - ( $a * 1000 ) ]
	set a [expr $val / 100 ]
	set digit [ lreplace $digit 	2 2 $a ]
	set val [ expr $val - ( $a * 100 ) ]
	set a [expr $val / 10 ]
	set digit [ lreplace $digit  1 1 $a ]
	set a [ expr $val - ( $a * 10 ) ]
	set digit [ lreplace $digit	0 0 $a ]
	return $digit
}
proc Digit2List { val } {
	set digit { 0 0 }
	set d2a [expr $val / 10 ]
	set digit [ lreplace $digit  1 1 $d2a ]
	set d2a [ expr $val - ( $d2a * 10 ) ]
	set digit [ lreplace $digit	0 0 $d2a ]
	return $digit
}

proc ExtractAddressed_Value {line } {
	global Prom_extra

	set a [ scan $line "%d %d" address data ]
	InsertLog "Got a data line : A= $address : D= $data"
	lappend Prom_extra $address $data 
}
proc ExtractString { line  } {
	set a [ string first ":" $line ]
	set b [ string length $line ]
	if { $a == -1 } { return -1 }
	set a [ expr $a + 1 ]
	return [ string range $line $a $b ]
}
proc ExtractChar { line  } {
	set b [ string length $line ]
	if { $b == -1 } { return -1 }
	set a [ expr $b - 1 ]
	return [ string range $line $a $b ]
}
proc SelectFile {args} {
    global env CurrentFile

    	option add *Canvas.background  #c0c020
	set types {
	{ {Text Files} {*.txt} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "Board Data file selection"  \
		  -initialdir [file dirname $CurrentFile] -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

      set CurrentFile $f
}

proc Prom_write { addr data } {
	global Prom_control Prom_status Prom_base Prom_limit
	global %%Waiting

# reset the failed flag regardless 
	Set_Register $Prom_control 1
# remove the write protect
	Set_Register $Prom_control 0x80
	Set_Register [ expr $Prom_base + $addr ] $data
	for { set i 0 } { $i < $Prom_limit } { incr i } {
		set a [ expr [ Get_Register $Prom_status ] & 1 ]
		if { $a == 0 } {
			break 
		}
	}
	if { $i == $Prom_limit } {
		InsertLog "Prom is busy longer than $Prom_limit "
		WaitForUser "Continue ? "
		set rtnd [ set %%Waiting ]
		if { $rtnd == -2 } {
			return $rtnd
		}
	}
	set a [ expr [ Get_Register $Prom_status ] & 8 ]
	if { $a != 0 } {
		InsertLog "Status reporting failed operation. "
		return -2 
	} else {
		return 0	
	}
}
# finally do the work ;-)
run_test

# WaitForUser "Data enabled, is the  ?"
#    syntax  Set_Register address_offset  value
# WaitForUser "Data synchronised ?"
# InsertLog "wait returned [set %%Waiting]"
# values of %%Waiting are  -1 (continue)   and   -2  (Abort)
# InsertLog "register at offset 2 has value [Get_Register 2]"
#    syntax  Get_Register address_offset
#    returns value

# potential Fpgas are 
#       "Core: Virtex for Segment ADC card 1" 
#       "Core: Virtex for Segment ADC card 2" 
#       "Core: Virtex for core ADCs" 
#       "Core: Main Board" 
#       "Segment: Virtex for Segment ADC card 1" 
#       "Segment: Virtex for Segment ADC card 2" 
#       "Segment: Virtex for Segment ADC card 3" 
#       "Segment: Virtex for Segment ADC card 4" 
#       "Segment: Main Board" 
