#    Diagnostic and test task for the AGATA Digitizer interface


set LogFlag 1

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"

set AgataServerList {}
set AgataServer "select a server"
variable WebServer

variable Temperature_Period

set ResourceList { \
       "Core: Virtex for Segment ADC card 1" \
       "Core: Virtex for Segment ADC card 2" \
       "Core: Virtex for core ADCs" \
       "Core: Main Board" \
       "Segment: Virtex for Segment ADC card 1" \
       "Segment: Virtex for Segment ADC card 2" \
       "Segment: Virtex for Segment ADC card 3" \
       "Segment: Virtex for Segment ADC card 4" \
       "Segment: Main Board" \
}
set ResourceIDs {0 1 2 3 8 9 10 11 12}


set FpgaList {}
set FpgaIDs {}
set Fpga ""

set AddressModifierList {A16 A32}
set AddressModifier [lindex $AddressModifierList 0]

set NumDigitizer -1
set DigitizerList {}

set Digitizer -1         ;#   currently selected digitizer
set CorePresent 0        ;#   for current digitizer
set SegmentPresent 0     ;#   for current digitizer
for {set i 0} {$i <= 12} {incr i} {set Present($i) 0}     ;# index is fpga ID

set SegmentTempList { \
       "Segment: Virtex for Segment ADC card 1" \
       "Segment: Analog for Segment ADC card 1" \
       "Segment: Virtex for Segment ADC card 2" \
       "Segment: Analog for Segment ADC card 2" \
       "Segment: Virtex for Segment ADC card 3" \
       "Segment: Analog for Segment ADC card 3" \
       "Segment: Virtex for Segment ADC card 4" \
       "Segment: Analog for Segment ADC card 4" \
       "Segment: PSU #1" \
       "Segment: PSU #2" \
}

set CoreTempList { \
       "Core: Virtex for Segment ADC card 1" \
       "Core: Analog for Segment ADC card 1" \
       "Core: Virtex for Segment ADC card 2" \
       "Core: Analog for Segment ADC card 2" \
       "Core: Virtex for core ADCs" \
       "Core: Analog for core ADCs" \
       "Core: PSU #1" \
       "Core: PSU #2" \
       "Core: PSU #3" \
}

    for {set fpga 0} {$fpga <= 12} {incr fpga} {
        for {set i 1} {$i <= 7} {incr i} {set Offset($fpga,$i) 0}
    }


set Address 0x0
set Data 0x0
set Length 1
set BlkData 0x0000

set ShortTimeOut 5000
set verbose 0


proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :toplevel T
         wm protocol $frame WM_DELETE_WINDOW "destroy $frame"

    :galley G :deaf T :strings $text :font fixed
    :show T
}

proc bgerror err {
    global errorInfo tkerrorInfo tkskiperror
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    if {![info exists FaultCode] || $FaultCode == "NONE"} {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$err" "error" 0 "continue" "Stack Trace"} button]
    } else {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo" \
                       "error" 0 "continue"} button]
    }
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}



proc make-frame {} {
   global env
   global AddressModifierList Address Data Length BlkData

set rows 16
set cols 80

  :frame [self frame-name] :label "[self frame-title]: Control & Diagnostics" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 0
    :text XPORTADDRT         :label "Agata Digitizer Web Service" :w 20 :value "select a server"
    :choice-stack XPORTADDRC :label "" :strings "xxxxxxxxx" 
    :hskip 100
    :checkbox VERBOSE        :label "Enable Verbose Option"
 
  :panel P3 :layout V :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 30
      :choice-stack DIGITIZER :label "Digitizer"  :exclusive T :menu "" :value -1
      :button PROBE           :label "Probe Hardware" 
      :choice-stack FPGA      :label "FPGA"  :exclusive T :strings "" :value -1
      :hskip 250
      :button PROM  :label "Read PROM"

  :panel P4 :layout V :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 15
      :choice-stack ADDRMOD   :label "Address Modifier"  :exclusive T :strings $AddressModifierList :value 0
      :text ADDR     :label "Address/Command" :w 10 :value $Address
      :text DATA     :label "Set Data"    :w 10 :value $Data
      :text LENGTH   :label "Read Block Length" :w 6 :value $Length
      :next-row T
      :text BLKDATA  :label "Write Block Data"  :w 80 :value $BlkData

  :panel P5 :layout V :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 30
      :button GET    :label "Do Get"  :w 14
      :button SET    :label "Do Set"  :w 14
      :button READ   :label "Do Read Block"  :w 14
      :button WRITE  :label "Do Write Block" :w 14

      :next-row T

      :button SHUTDOWN :label "Shutdown Power" 
      :button STATUS   :label "Show Status Registers"
      :button INTERNAL :label "Select Internal Clock"
      :button EXTERNAL :label "Select External Clock"

      :next-row T
      :col-gap 15

      :slider OFFSET1 :label "Offset Ch1" :min 0 :max 65535 :value 0
      :slider OFFSET3 :label "Offset Ch3" :min 0 :max 65535 :value 0
      :slider OFFSET5 :label "Offset Ch5" :min 0 :max 65535 :value 0

      :next-row T

      :slider OFFSET2 :label "Offset Ch2" :min 0 :max 65535 :value 0
      :slider OFFSET4 :label "Offset Ch4" :min 0 :max 65535 :value 0
      :slider OFFSET6 :label "Offset Ch6" :min 0 :max 65535 :value 0

      :next-row T

      :menubutton OFFSETS :label "Offsets Actions" :strings [list "Set Current FPGA Offsets" "Set All FPGA Offsets" "Read Current FPGA Offsets" \
             "Read All FPGA Offsets" "Save Offsets" "Restore Offsets"]

      :hskip 450
      :slider OFFSET7 :label "Offset Ch7" :min 0 :max 65535 :value 0

      :next-row T

      :menubutton SHOW :label "Show Actions" :strings [list "Show Current FPGA Status register" ]

      :hskip 200
      :button SETUPALL    :label "SetUp Digitizer" 


  :panel P2 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font fixed
     configuretags

  :panel P1 :layout V :padx 5 :pady 5
    :layout H; :col-gap 30
     :button CLEAR   :label "Empty Log Window" :command ":modify LOG :empty T"

  :panel P7 :layout V :padx 5 :pady 5
    :layout H; :col-gap 30
     :button COMMS   :label "Data Communications Control" :w 32 :command Data_Comms
     :menubutton TEMP    :label "Temperature Monitoring"   :strings [list text graphical] 
     :button TESTS   :label "Run Tests"                   :w 32 :command Hardware_Tests

   open-frame

  :galley_size LOG $rows $cols

#    focus [:path CMD].entry

   :show T
}


proc click-configure {W} {:galley_resize LOG}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n}

proc start args {
   global started env Options AgataServerList

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1

#    load application options and establish default values for any not defined

   load_Options

   if {![info exists Options(WebServer)]}           {set Options(WebServer) "localhost"}
   if {![info exists Options(Temperature_Period)]}  {set Options(Temperature_Period) 5000}

   update_Options

   foreach option [list WebServer Temperature_Period] {
       global $option
       set $option $Options($option)
   }

   set  AgataServerList [list $WebServer]    ;# put default into the menu
   :modify XPORTADDRC :strings $AgataServerList :value [lsearch $AgataServerList $WebServer]
   
}

proc Save_Options {} {
   global Options

   foreach option [list WebServer Temperature_Period] {
       global $option
       set Options($option) [set $option]
   }

   update_Options
}

proc click arg {
   global history verbose longbuffer datafile usedatafile
   global NumDigitizer Digitizer

   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer

     ;# ensure a server has been selected
   switch -glob $w  {

      XPORTADDRT    -
      XPORTADDRC    -
      VERBOSE       {}
   
      default       {
           if {$NumDigitizer == -1} {
               midas-warning "Please select an AGATA Digitizer Web Service"
               return
           }
      }
   }

     ;# ensure a digitizer has been selected
   switch -glob $w  {

      XPORTADDRT    -
      XPORTADDRC    -
      VERBOSE       -
      DIGITIZER     {}

      default   {
           if {$Digitizer == -1} {
               midas-warning "Please select a Digitizer"
               return
           }
      }
   }

   switch -glob $w  {

      XPORTADDRT    {click-server $v}
      XPORTADDRC    {click-server $i}
      
      VERBOSE       {set verbose $v; AGATADigitizerAccessClient__AccessServerVerbose $verbose}

      DIGITIZER     {click-digitizer $i}

      FPGA          {click-fpga $i}
      SHOW       {
          switch $i {
             0     {do_showStatus}
             1     {}
             2     {}
             3     {}
             4     {}
             5     {}
          }
      }
      TEMP       {Monitor_Temperature $i}
      OFFSETS       {
          switch $i {
             0     {do_setcurrentfpgaoffsets}
             1     {do_setallfpgaoffsets}
             2     {do_getcurrentfpgaoffsets}
             3     {do_getallfpgaoffsets}
             4     {do_saveoffsets}
             5     {do_restoreoffsets}
          }
      }
      OFFSET*      {click-offset [string range $w 6 6] $v}
      PROM         {click__readprom}
      SETUP        {click__setup}
 
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc click-server {v} {
    global env AgataServerList WebServer AGATADigitizerAccessServerHost
    global NumDigitizer DigitizerList Digitizer
    global verbose

    if {$v == 0} {set WebServer [first $AgataServerList]} else {set WebServer $v}
    set AgataServerList [list $WebServer]    ;# put default into the menu
    :modify XPORTADDRT :value $WebServer
    :modify XPORTADDRC :strings $AgataServerList :value [lsearch $AgataServerList $WebServer]

    Save_Options

    set AGATADigitizerAccessServerHost $WebServer
    package require AGATADigitizerAccessClient

    loadsource AGATADigitizerAccessClient     ;# better solution needed

    AGATADigitizerAccessClient__AccessServerVerbose $verbose
    set DigitizerList [AGATADigitizerAccessClient__Init]
    set NumDigitizer [llength $DigitizerList]

    insert "connected to Agata Digitizer Web Service at host $WebServer: $NumDigitizer digitizer(s) discovered ($DigitizerList)"

    :modify DIGITIZER :menu $DigitizerList :value -1
    
    if {[llength $DigitizerList] == 1} {click-digitizer [first $DigitizerList]} else {insert "select a digitizer"}
}

proc click-data {v} {
    global Data

    set Data $v

    insert "Data= [format "0x%x" $Data]"
}

proc click-blkdata {v} {
    global BlkData

    set BlkData $v
}

proc click-length {v} {
    global Length

    set Length $v

    insert "Read Block Length= [format "0x%x" $Length]"
}


proc click-addr {v} {
    global Address

    set Address $v

    insert "Address/Cmd= [format "0x%x" $Address]"
}

proc click-offset {i vl} {
    global Offset
    global FpgaList FpgaIDs Fpga

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set Offset($fpga,$i) $vl
}

proc click-probe {args} {
    global Digitizer

    click-digitizer $Digitizer
}

proc click-digitizer {v} {
    global Digitizer DigitizerList Present CorePresent SegmentPresent
    global FpgaList FpgaIDs
    global ShortTimeOut

    set Digitizer $v
    set CorePresent 0
    set SegmentPresent 0
    for {set i 0} {$i <= 12} {incr i} {set Present($i) 0} 

    insert "Digitizer= $Digitizer"

    set FpgaList {}
    set FpgaIDs {}

    insert "probing for Core Main Board"

    set fpga 3     ;# core main board
    set cmd 14     ;# safe read command for test
    set Length 6   ;# returns 6 bytes
    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $cmd A16 $Length} m]
    if {$rc == 0} {
        insert "found it"
        lappend FpgaList "Core: Main Board"
        lappend FpgaIDs $fpga
        set CorePresent 1
        set Present($fpga) 1
    } else {
        insert "Error returned [first [split $m \n]]"
     }

    if {$CorePresent == 1} {

       insert "probing for Core ADCs"

       set fpga 2    ;# core ADCs
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Core: Virtex for core ADCs"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

       insert "probing for Core segment ADC card #1"

       set fpga 0    ;# core segment ADC card #1
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Core: Virtex for Segment ADC card 1"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

       insert "probing for Core segment ADC card #2"

       set fpga 1    ;# core segment ADC card #2
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Core: Virtex for Segment ADC card 2"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }
    }

    insert "probing for Segment Main Board"

    set fpga 12    ;# segment main board
    set cmd 14     ;# safe read command for test
    set Length 6   ;# returns 6 bytes
    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $cmd A16 $Length} m]
    if {$rc == 0} {
        insert "found it"
        lappend FpgaList "Segment: Main Board"
        lappend FpgaIDs $fpga
        set SegmentPresent 1
        set Present($fpga) 1
    } else {
        insert "Error returned [first [split $m \n]]"
    }

    if {$SegmentPresent == 1} {

       insert "probing for Segment ADC card #1"

       set fpga 8    ;# Segment ADC card #1
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Segment: Virtex for Segment ADC card 1"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

       insert "probing for Segment ADC card #2"

       set fpga 9    ;# Segment ADC card #2
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Segment: Virtex for Segment ADC card 2"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

       insert "probing for Segment ADC card #3"

       set fpga 10    ;# Segment ADC card #3
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Segment: Virtex for Segment ADC card 3"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }

       insert "probing for Segment ADC card #4"

       set fpga 11    ;# Segment ADC card #4
       set address 15
       set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
       set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $address A16} m]
       if {$rc == 0} {
           insert "found it"
           lappend FpgaList "Segment: Virtex for Segment ADC card 4"
           lappend FpgaIDs $fpga
           set Present($fpga) 1
       } else {
           insert "Error returned [first [split $m \n]]"
       }
   }

   set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer 0} m]   ;# reset timeout to default

   if {[expr $CorePresent + $SegmentPresent] == 0} {
       insert "Digitizer not found"
       :modify DIGITIZER  :value -1
       :modify FPGA :strings "" :value -1
   } else {
       :modify DIGITIZER :value [lsearch $DigitizerList $Digitizer]
       :modify FPGA :strings $FpgaList :value 0
       click-fpga [first $FpgaList]
   }

}

proc click__readprom {} {
    global Digitizer FpgaList FpgaIDs Fpga

set Prom_control 0x880
set Prom_status 0x881
set Prom_base 0x800

        if {$Digitizer == -1} {
            insert "Use \"Probe Hardware\" and select a Digitizer"
            return
        }

        if {$Fpga == "Use Probe Hardware"} {
            insert "Use \"Probe Hardware\" and select an FPGA"
            return
        }

    insert "Digitizer= $Digitizer"

    insert "Read PROM"

#    not available for Main Boards

        if {$Fpga == "Core: Main Board" || $Fpga == "Segment: Main Board"} {
            insert "Not available for Main Boards"
            return
        }

# check to see if it has read in correctly 

        set a [ Get__Register $Prom_status ]
	if { $a != 4 } {
             insert "Prom has read back unsuccessfully. Status : $a"
             return
        }

	insert "Prom Read back into RAM"

	set a [ format "%c" [ Get__Register 0x800 ] ]
	
	if { $a == "C" } {
		insert "This is a Core ADC board"
	} elseif { $a == "S" } {
		insert "This is a Segment ADC board" 
	} else {
		insert "Not sure what this is : [ format "%c" $a ] !"
	}
	set a [ Get__Register 0x801 ]
	set b [ Get__Register 0x802 ]
	set c [ Get__Register 0x803 ]
	set d [ Get__Register 0x804 ]
	set e [ Get__Register 0x805 ]
	set f [ Get__Register 0x806 ]
	set g [ Get__Register 0x807 ]
	set serial [ format "%c%c%c%c" $d $c $b $a ]
	set revision [ format "%c" $e ]
	set mod_state [ format "%c%c" $g $f ]

	insert "Serial number : $serial"
	insert "Revision      : $revision"
	insert "Mod state     : $mod_state"
	insert "Test Complete ....."

}

proc click__setup {} {
    global Digitizer FpgaList FpgaIDs Fpga

# set the external clock option
	if { [Select__Fpga "Core: Main Board"] == -1 } then { 
		insert "Can't connect to Main board" 
		return
	}
	set rtnd 0
	Set__Register 40 0 

# enable the Core Laser and push it's DCMs to lock.

	if { [Select__Fpga "Core: Virtex for core ADCs"] == -1 } then { 
		insert "No Core card available !" 
		return
	}

# set the sync pulse to be correctly transmitted

	Set__Register 11 1
	
# set the basics - inpsection and main enables
	Set__Register 4 0

	Set__Register 7 0x300
	Set__Register 0 0x175
	Set__Register 0 0x75
 	set val [Get__Register 1 ]
	set val [ expr $val & 0xfffd ]
 	if { $val != 0xf205 } then {
		insert "Core status  fault : ([format "0x%x" $val])"
		insert "Is the cable connected ?"
	}
	setup__core  
	setup__segment "Core: Virtex for Segment ADC card 1" 
	setup__segment "Core: Virtex for Segment ADC card 2" 
	setup__segment "Segment: Virtex for Segment ADC card 1" 
	setup__segment "Segment: Virtex for Segment ADC card 2" 
	setup__segment "Segment: Virtex for Segment ADC card 3" 
	setup__segment "Segment: Virtex for Segment ADC card 4" 

      insert "Setup Complete ....."
}

proc click__setupfpga {} {
    global Digitizer FpgaList FpgaIDs Fpga

        if {$Fpga == "Core: Main Board" || $Fpga == "Segment: Main Board"} {
            insert "Not available for Main Boards"
            return
        }

        if {$Fpga == "Core: Virtex for core ADCs"} {setup__core} else {setup__segment $Fpga}

        insert "Setup $Fpga Complete ....."

}

proc setup__segment { board } {
    global Digitizer FpgaList FpgaIDs Fpga

	set rtnd 0

	if { [Select__Fpga "$board"] == -1 } then { 
		insert "Segment card --- %board---- not available !" 
		return
	}
# set the basics  disable inspection and main enables
	setup__offset_seg
# enable the laser
	Set__Register 0 5
	Set__Register 4 0
	Set__Register 7 0xFF00

# check the status register for DCM locked.
	set val [Get__Register 1 ]
	if { $val != 0xe001 } {
		insert "Segment status  fault : ([format "0x%x" $val])"
	        return
	}
# channel by channel set transmit data enabled
	foreach val { 1 2 3 4 5 6 } {
		set ch_rkt [expr ( $val << 4 ) + 3 ]
		Set__Register $ch_rkt 0x7c00
		set ch_rkt [expr ( $val << 4 ) + 4 ]
		Set__Register $ch_rkt 0
	}
# aving these as two separate or statements allows link synch.
	foreach val { 1 2 3 4 5 6 } {
		set ch_rkt [expr ( $val << 4 ) + 3 ]
		Set__Register $ch_rkt 0
	}
	
}

proc setup__core {} {
    global Digitizer FpgaList FpgaIDs Fpga

	if { [Select__Fpga "Core: Virtex for core ADCs"] == -1 } then { 
		WaitForUser "No Core card available !" 
		return
	}
# set the basics - inspection and main enables
	setup__offset_core
	Set__Register 4 0
	Set__Register 7 0x300

# channel by channel set transmit data enabled
	foreach val { 1 2  } {
		set ch_rkt [expr ( $val << 4 ) + 3 ]
		Set__Register $ch_rkt 0x7c00
		set ch_rkt [expr ( $val << 4 ) + 4 ]
		Set__Register $ch_rkt 0
	}
	foreach val { 1 2  } {
		set ch_rkt [expr ( $val << 4 ) + 3 ]
		Set__Register $ch_rkt 0
	}
}

proc setup__offset_core {} {
    global Digitizer FpgaList FpgaIDs Fpga
    global Offset

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        for {set i 1} {$i <= 2} {incr i} {Set__Register [expr 0x80 + $i] [set Offset($fpga,$i)]; insert "Set Offset $i = [set Offset($fpga,$i)]"}
}

proc setup__offset_seg {} {
    global Digitizer FpgaList FpgaIDs Fpga
    global Offset

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        for {set i 1} {$i <= 7} {incr i} {Set__Register [expr 0x80 + $i] [set Offset($fpga,$i)]; insert "Set Offset $i = [set Offset($fpga,$i)]"}
}

proc do_getcurrentfpgaoffsets {} {
    global FpgaList FpgaIDs Fpga
    global Offset

        if {$Fpga == "Core: Main Board" || $Fpga == "Segment: Main Board"} {
            insert "Not available for Main Boards"
            return
        }

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        if {$Fpga == "Core: Virtex for core ADCs"} {
            for {set i 1} {$i <= 2} {incr i} {
                set Offset($fpga,$i) [Get__Register [expr 0x80 + $i]]
            }
            for {set 3 1} {$i <= 7} {incr i} {set Offset($fpga,$i) 0}
        } else {
            for {set i 1} {$i <= 7} {incr i} {
                set Offset($fpga,$i) [Get__Register [expr 0x80 + $i]]
            }
        }

        click-fpga $Fpga
}

proc do_setcurrentfpgaoffsets {} {
    global FpgaList FpgaIDs Fpga
    global Offset

        if {$Fpga == "Core: Main Board" || $Fpga == "Segment: Main Board"} {
            insert "Not available for Main Boards"
            return
        }

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        if {$Fpga == "Core: Virtex for core ADCs"} {
            for {set i 1} {$i <= 2} {incr i} {
                Set__Register [expr 0x80 + $i] [set Offset($fpga,$i)]
            }
        } else {
            for {set i 1} {$i <= 7} {incr i} {
                Set__Register [expr 0x80 + $i] [set Offset($fpga,$i)]
            }
        }

        click-fpga $Fpga
}

proc do_getallfpgaoffsets {} {
    global FpgaList FpgaIDs Fpga
    global Offset

    set KEPT $Fpga

    foreach Fpga $FpgaList {

        if {$Fpga == "Core: Main Board" || $Fpga == "Segment: Main Board"} {continue}

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        if {$Fpga == "Core: Virtex for core ADCs"} {
            for {set i 1} {$i <= 2} {incr i} {
                set Offset($fpga,$i) [Get__Register [expr 0x80 + $i]]
            }
            for {set 3 1} {$i <= 7} {incr i} {set Offset($fpga,$i) 0}
        } else {
            for {set i 1} {$i <= 7} {incr i} {
                set Offset($fpga,$i) [Get__Register [expr 0x80 + $i]]
            }
        }
   }

   click-fpga $KEPT
 
}

proc do_setallfpgaoffsets {} {
    global FpgaList FpgaIDs Fpga
    global Offset

    set KEPT $Fpga

    foreach Fpga $FpgaList {

        if {$Fpga == "Core: Main Board" || $Fpga == "Segment: Main Board"} {continue}

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        if {$Fpga == "Core: Virtex for core ADCs"} {
            for {set i 1} {$i <= 2} {incr i} {
                Set__Register [expr 0x80 + $i] [set Offset($fpga,$i)]
            }
        } else {
            for {set i 1} {$i <= 7} {incr i} {
                Set__Register [expr 0x80 + $i] [set Offset($fpga,$i)]
            }
        }
   }

   click-fpga $KEPT
 
}

proc do_saveoffsets {} {
    global FpgaList FpgaIDs Fpga
    global Offset
    global Options

    set KEPT $Fpga

    foreach Fpga $FpgaList {

        if {$Fpga == "Core: Main Board" || $Fpga == "Segment: Main Board"} {continue}

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        if {$Fpga == "Core: Virtex for core ADCs"} {
            for {set i 1} {$i <= 2} {incr i} {
                set Options(Offset($fpga,$i)) [set Offset($fpga,$i)]
            }
        } else {
            for {set i 1} {$i <= 7} {incr i} {
                set Options(Offset($fpga,$i)) [set Offset($fpga,$i)]
            }
        }
   }

   click-fpga $KEPT

    update_Options
 
}

proc do_restoreoffsets {} {
    global FpgaList FpgaIDs Fpga
    global Offset
    global Options

    set KEPT $Fpga

   load_Options

    foreach Fpga $FpgaList {

        if {$Fpga == "Core: Main Board" || $Fpga == "Segment: Main Board"} {continue}

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        if {$Fpga == "Core: Virtex for core ADCs"} {
            for {set i 1} {$i <= 2} {incr i} {
                if {[info exists Options(Offset($fpga,$i))]} {set Offset($fpga,$i) [set Options(Offset($fpga,$i))]}
            }
        } else {
            for {set i 1} {$i <= 7} {incr i} {
                if {[info exists Options(Offset($fpga,$i))]} {set Offset($fpga,$i) [set Options(Offset($fpga,$i))]}
            }
        }
   }

   click-fpga $KEPT
 
}

proc click-fpga {v} {
    global FpgaList FpgaIDs Fpga
    global Offset

    set Fpga $v

    insert "Fpga= $Fpga"

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]
  
    for {set i 1} {$i <= 7} {incr i} {:modify OFFSET$i :value [set Offset($fpga,$i)]}
}

proc click-addrmod {v} {
    global AddressModifierList AddressModifier

    set AddressModifier [lindex $AddressModifierList $v]

    insert "AddressModifier= $AddressModifier"
}

proc do_showStatus {} {
    global FpgaList FpgaIDs Fpga

        if {$Fpga == "Core: Main Board" || $Fpga == "Segment: Main Board"} {
            insert "Not available for Main Boards"
            return
        }

    set status [Get__Register 1]

    set info "$Fpga\n\n"

    if {[string range $Fpga 0 3] == "Core"} {
      for {set i 0} {$i <= 15} {incr i} {
        set bit [expr ($status >> $i) & 1]
        switch $i {
            0  {append info "[format "%-50s %-10s" "Laser Fault: fault when value = 0" $bit]\n"}
            1  {append info "[format "%-50s %-10s" "Laser Signal Detect" $bit]\n"}
            2  {append info "[format "%-50s %-10s" "Sync OK: SYNC pulse received OK when value = 1" $bit]\n"}
            3  {append info "[format "%-50s %-10s" "spare" $bit]\n"}
            4  {append info "[format "%-50s %-10s" "RAM test passed" $bit]\n"}
            5  {append info "[format "%-50s %-10s" "RAM test failed" $bit]\n"}
            6  {append info "[format "%-50s %-10s" "RAM test in progress" $bit]\n"}
            7  {append info "[format "%-50s %-10s" "spare" $bit]\n"}
            8  {append info "[format "%-50s %-10s" "TX1 Polarity setting" $bit]\n"}
            9  {append info "[format "%-50s %-10s" "TX2 Polarity setting" $bit]\n"}
            10 {append info "[format "%-50s %-10s" "Control Clock select from Spartan" $bit]\n"}
            11 {append info "[format "%-50s %-10s" "spare" $bit]\n"}
            12 {append info "[format "%-50s %-10s" "Shifter clock DCM Lock signal state: locked = 1" $bit]\n"}
            13 {append info "[format "%-50s %-10s" "Laser clock DCM Lock signal state: locked = 1" $bit]\n"}
            14 {append info "[format "%-50s %-10s" "Slow Interface DCM Lock signal state: locked = 1" $bit]\n"}
            15 {append info "[format "%-50s %-10s" "Top BREF DCM Lock signal state: locked = 1" $bit]\n"}
        }
      }
    } else {
      for {set i 0} {$i <= 15} {incr i} {
        set bit [expr ($status >> $i) & 1]
        switch $i {
            0  {append info "[format "%-50s %-10s" "Laser Fault: fault when value = 0" $bit]\n"}
            1  {append info "[format "%-50s %-10s" "spare" $bit]\n"}
            2  {append info "[format "%-50s %-10s" "spare" $bit]\n"}
            3  {append info "[format "%-50s %-10s" "spare" $bit]\n"}
            4  {append info "[format "%-50s %-10s" "RAM test passed" $bit]\n"}
            5  {append info "[format "%-50s %-10s" "RAM test failed" $bit]\n"}
            6  {append info "[format "%-50s %-10s" "RAM test in progress" $bit]\n"}
            7  {append info "[format "%-50s %-10s" "spare" $bit]\n"}
            8  {append info "[format "%-50s %-10s" "TX1 Polarity setting" $bit]\n"}
            9  {append info "[format "%-50s %-10s" "TX2 Polarity setting" $bit]\n"}
            10 {append info "[format "%-50s %-10s" "TX3 Polarity setting" $bit]\n"}
            11 {append info "[format "%-50s %-10s" "TX4 Polarity setting" $bit]\n"}
            12 {append info "[format "%-50s %-10s" "TX5 Polarity setting" $bit]\n"}
            13 {append info "[format "%-50s %-10s" "TX6 Polarity setting" $bit]\n"}
            14 {append info "[format "%-50s %-10s" "Slow Interface DCM Lock signal state: locked = 1" $bit]\n"}
            15 {append info "[format "%-50s %-10s" "Top BREF DCM Lock signal state: locked = 1" $bit]\n"}
        }
      }
    }

    midas-information $info
 
}

proc click-status {args} {
    global Digitizer FpgaList FpgaIDs Fpga
    global ShortTimeOut
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]
    if {$fpga < 8} then {set fpga 3} else {set fpga 12} ;# select main board 

    set cmd 14     ;# read status registers command
    set Length 6   ;# returns 6 bytes
    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $cmd A16 $Length} m]
    if {$rc == 0} {
        insert "status registers: $m"
    } else {
        insert "Read returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    }
}

proc click-clock {vl} {
    global Digitizer FpgaList FpgaIDs Fpga
    global ShortTimeOut
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga 3       ;# available to core main board only
    set cmd 40 

    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Set $Digitizer $fpga $cmd A16 $vl} m]
    if {$rc == 0} {
        insert "Done"
    } else {
        insert "Set returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    }
}

proc click-internal {args} {click-clock 0x0100}
proc click-external {args} {click-clock 0x0000}

proc click-shutdown {args} {
    global Digitizer FpgaList FpgaIDs Fpga
    global ShortTimeOut
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]
    if {$fpga < 8} then {set fpga 3} else {set fpga 12} ;# select main board 

    set cmd 20       ;# shutdown command

    set rc [catch {AGATADigitizerAccessClient__SetTimeOut $Digitizer  $ShortTimeOut} m]
    set rc [catch {AGATADigitizerAccessClient__Set $Digitizer $fpga $cmd A16 0x0800} m]
    if {$rc == 0} {
        insert "Done"
    } else {
        insert "Set returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    }
}

proc click-set {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address AddressModifier Data
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Set $Digitizer $fpga $Address $AddressModifier $Data} m]
    if {$rc != 0} {
        insert "Set returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    } else {
        insert "Set returned $m ([format "0x%x" $m])"
    }
}

proc click-get {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address AddressModifier
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Get $Digitizer $fpga $Address $AddressModifier} m]
    if {$rc != 0} {
        insert "Get returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
        return -1
    } else {
        insert "Get returned $m ([format "0x%x" $m])"
        return $m
    }
}

proc click-read {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address Length AddressModifier
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $Address $AddressModifier $Length} m]
    if {$rc != 0} {
        insert "Read returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    } else {
        insert "Read returned $m"
    }
}

proc click-write {args} {
    global Digitizer FpgaList FpgaIDs Fpga Address Length AddressModifier BlkData
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

    set rc [catch {AGATADigitizerAccessClient__Write $Digitizer $fpga $Address $AddressModifier [llength $BlkData] $BlkData} m]
    if {$rc != 0} {
        insert "Write returned with an error\n$m\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo"
    } else {
        insert "Write returned $m"
    }
}

proc Set__Register {Addr Vl {Space A16}}  {
    global Digitizer FpgaList FpgaIDs Fpga Address Length AddressModifier
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

        set fpga [lindex $FpgaIDs [lsearch $FpgaList [set Fpga]]]

        set rc [catch {AGATADigitizerAccessClient__Set [set Digitizer] $fpga $Addr $Space $Vl} m]
        if {$rc != 0} {
            insert  "Set returned with an error\n$m\n$errorCode\n$errorInfo"
            return -1
        } else {
            return $m
        }
}

proc Get__Register {Addr {Space A16}}  {
    global Digitizer FpgaList FpgaIDs Fpga Address Length AddressModifier
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

        set fpga [lindex $FpgaIDs [lsearch $FpgaList [set Fpga]]]

        set rc [catch {AGATADigitizerAccessClient__Get [set Digitizer] $fpga $Addr $Space} m]
        if {$rc != 0} {
            insert  "Get returned with an error\n$m\n$ErrorCode\n$ErrorInfo"
            return -1
        } else {
            return $m
        }
}

proc Select__Fpga {n} {
    global Digitizer FpgaList FpgaIDs Fpga Address Length AddressModifier

   if {[lsearch $FpgaList $n] == -1 || [lindex $FpgaIDs [lsearch $FpgaList $n]] == -1} {
       insert "$n is a bad Fpga"
       return -1
   } else {
       set Fpga $n
       :modify FPGA :value [lsearch $FpgaList [set Fpga]]
   }
   return [set Fpga]
}

