#    Diagnostic and test task for the AGATA Digitizer interface

#    Temperature component 


set Run 0    ;#  temperature monitor

set NumRows 5   ;#   for temperature monitor layout
set NumCols 4
set XSize 300
set YSize 150

set Mode 0     ;#   text=0; graphical=1
 
proc Monitor_Temperature {i} {
   global env
   global ResourceList
   global NumRows NumCols
   global XSize YSize
   global Mode

   set Mode $i

  :frame [self frame-name].temp :label "[self frame-title]: Temperature Monitoring" :icon-name "[self icon-title]"

  if {$Mode == 0} {

       :panel P0 :layout V 
       :galley C0 :deaf T :font fixed :rows 22

       :modify C0  :createtag "high  -foreground red -background black"
       :modify C0  :createtag "low   -foreground blue -background black"
       :modify C0  :createtag "ok    -foreground green -background black"
       :modify C0  :createtag "white -foreground white -background black"

  }

  if {$Mode == 1} {

    for {set i 0} {$i < $NumRows} {incr i} {

       :panel P$i :layout V :padx 0 :pady 0
         :layout H

       for {set j 0} {$j < $NumCols} {incr j} {

         set c [expr ($i*$NumCols)+$j]
         set resource [lindex $ResourceList $c]
         :canvas C$c :w $XSize :h $YSize
            [:path C$c] configure -background black
       }
    }
  }

    :panel LAST :layout V :padx 5 :pady 5
    :layout H; :col-gap 20

      :button DONE   :label "Done"  :command "frame-quit [self frame-name].temp"
      :button TSTART :label "Start" :command Start_TempMon
      :button TSTOP  :label "Stop"  :command Stop_TempMon
      :button RESET  :label "Reset" :command Reset_TempMon

    :show T

   Reset_TempMon
   Stop_TempMon

}

proc Reset_TempMon {} {
    global CoreTemperature SegmentTemperature
    global XSize YSize

    for {set i 0} {$i <= 8} {incr i} {
        set CoreTemperature($i) {}
        for {set j 0} {$j < $XSize} {incr j} {lappend CoreTemperature($i) 20.0}
    }

    for {set i 0} {$i <= 9} {incr i} {
        set SegmentTemperature($i) {}
        for {set j 0} {$j < $XSize} {incr j} {lappend SegmentTemperature($i) 20.0}
    }

}

proc Start_TempMon {} {
    global Run
 
    set Run 1
    Update_Temperature
}

proc Stop_TempMon {} {
    global Run
 
    set Run 0
}

proc ForTesting {} {
    global Present CorePresent SegmentPresent

   for {set i 0} {$i <= 12} {incr i} {
      set Present($i) 1
   }

  set CorePresent 1
  set SegmentPresent 1
}

proc Update_Temperature {} {
    global Digitizer
    global CorePresent CoreTemperature
    global SegmentPresent SegmentTemperature
    global Run
    global Options

    set-footer "Updating temperatures..."

    set Error 0

    if {$CorePresent == 1} {

       set fpga 3         ;# core main board
       set cmd 19         ;# use command 19
       set Length 20      ;# which returns 20 bytes of data
       set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $cmd A16 $Length} m]
       if {$rc == 0} {
#            insert "$m"
            set T [format_readings $m]
            for {set i 0} {$i <= 8} {incr i} {
                set CoreTemperature($i) [linsert [lreplace $CoreTemperature($i) end end] 0 [lindex $T $i]]
            }
       } else {
           insert "Error returned $m"
           incr Error
       }
    }

    if {$SegmentPresent == 1} {

       set fpga 12        ;# segment main board
       set cmd 19         ;# use command 19
       set Length 20      ;# which returns 20 bytes of data
       set rc [catch {AGATADigitizerAccessClient__Read $Digitizer $fpga $cmd A16 $Length} m]
       if {$rc == 0} {
#            insert "$m"
            set T [format_readings $m]
            for {set i 0} {$i <= 9} {incr i} {
                set SegmentTemperature($i) [linsert [lreplace $SegmentTemperature($i) end end] 0 [lindex $T $i]]
            }
       } else {
           insert "Error returned $m"
           incr Error
       }
    }

    clear-footer

    if {$Error == 0 && $Run == 1} {after $Options(Temperature_Period) Update_Temperature}

    Update_Graph

}

proc format_readings {Temps} {

    set T {}
    for {set i 0; set j 0} {$i < 20} {incr i; incr j} {
        set msb [lindex $Temps $i]
        incr i
        set lsb [lindex $Temps $i]
        set t [expr (((($msb * 255) + $lsb)/8) & 0x0fff) * 0.0625]
        if {$msb >= 128} then {
           lappend T [expr $t * -1.0]
        } else {
           lappend T $t 
        }
    }
#insert "[lindex $T 4] [lindex $T 5] [lindex $T 6] [lindex $T 7] [lindex $T 8]"
    return $T
}

proc Update_Graph {} {
    global Digitizer
    global CoreTemperature SegmentTemperature
    global Present

    show_reset

#    do Core

    if {$Present(0) == 1} {            ;# Segment ADC board 1 present
        show_core 0                     ;#  virtex
        show_core 1                     ;#  analog
    }
    if {$Present(1) == 1} {            ;# Segment ADC board 1 present
        show_core 2                     ;#  virtex
        show_core 3                     ;#  analog
    }
    if {$Present(2) == 1} {           ;# Core ADC board present 
        show_core 4                     ;#  virtex
        show_core 5                     ;#  analog
    }
    if {$Present(3) == 1} {           ;# core board 
        show_core 6                     ;#  psu #1
        show_core 7                     ;#  psu #2
        show_core 8                     ;#  psu #3
    }

#    do Segment

    if {$Present(8) == 1} {            ;# Segment ADC board 1 present 
        show_segment 0                     ;#  virtex
        show_segment 1                     ;#  analog
    }
    if {$Present(9) == 1} {            ;# Segment ADC board 2 present 
        show_segment 2                     ;#  virtex
        show_segment 3                     ;#  analog
    }
    if {$Present(10) == 1} {           ;# Segment ADC board 3 present 
        show_segment 4                     ;#  virtex
        show_segment 5                     ;#  analog
    }
    if {$Present(11) == 1} {           ;# Segment ADC board 4 present 
        show_segment 6                     ;#  virtex
        show_segment 7                     ;#  analog
    }
    if {$Present(12) == 1} {            ;# Core board present 
        show_segment 8                     ;#  psu #1
        show_segment 9                     ;#  psu #2
    }

}

proc show_reset {} {
    global Canvas
   global NumRows NumCols
    global Mode

  if {$Mode == 0} {      ;#   text

        :modify C0 :empty T
        :modify C0 :usingtag white :strings "Updated [date]\n\n"

  } else {

    for {set i 0} {$i < [expr $NumRows * $NumCols]} {incr i} {
        set _canvas [self frame-name].temp.p[expr $i/$NumCols].c$i
        :draw modify :tag t$i :delete T
        :draw modify :tag l$i :delete T
    }

    set Canvas 0
  }
}

set LowT 20
set HighT 40

proc show_segment {n} {
    global SegmentTemperature SegmentTempList
    global _canvas
    global Canvas
   global NumRows NumCols
   global XSize YSize
    global Mode
   global LowT HighT

  if {$Mode == 0} {     ;#   text

    set t [lindex $SegmentTemperature($n) 0]
    set color ok
    if {$t < $LowT}  {set color low}
    if {$t > $HighT} {set color high}
    :modify C0 :usingtag $color :strings "[format "%-50s %-4.1f" "[lindex $SegmentTempList $n]" "$t"]\n"

  } else {

    set _canvas [self frame-name].temp.p[expr $Canvas/$NumCols].c$Canvas

    :draw modify :tag t$n :delete T
    :draw modify :tag l$n :delete T

    for {set i 0; set j 1} {$i < $XSize} {incr i; incr j} {
        set t0 [lindex $SegmentTemperature($n) $i]
        set t0 [expr int ($YSize - (($t0 - 20) * 5))]
        set t1 [lindex $SegmentTemperature($n) $j]
        set t1 [expr int ($YSize - (($t1 - 20) * 5))]
        :draw line :tag l$n :colour red :value $i $t0 $j $t1
    }

    :draw text :tag t$n :colour white :text "[lindex $SegmentTempList $n]" :value 5 5

    incr Canvas
  }
}

proc show_core {n} {
    global CoreTemperature CoreTempList
    global _canvas
    global Canvas
   global NumRows NumCols
   global XSize YSize
    global Mode
   global LowT HighT

  if {$Mode == 0} {     ;#   text

    set t [lindex $CoreTemperature($n) 0]
    set color ok
    if {$t < $LowT}  {set color low}
    if {$t > $HighT} {set color high}
    :modify C0 :usingtag $color :strings "[format "%-50s %-4.1f" "[lindex $CoreTempList $n]" "$t"]\n"

  } else {

    set _canvas [self frame-name].temp.p[expr $Canvas/$NumCols].c$Canvas

    :draw modify :tag t$n :delete T
    :draw modify :tag l$n :delete T

    for {set i 0; set j 1} {$i < $XSize} {incr i; incr j} {
        set t0 [lindex $CoreTemperature($n) $i]
        set t0 [expr int ($YSize - (($t0 - 20) * 5))]
        set t1 [lindex $CoreTemperature($n) $j]
        set t1 [expr int ($YSize - (($t1 - 20) * 5))]
        :draw line :tag l$n :colour yellow :value $i $t0 $j $t1
    }

    :draw text :tag t$n :colour white :text "[lindex $CoreTempList $n]" :value 5 5

    incr Canvas
  }
}
