#    Diagnostic and test task for the AGATA Digitizer interface

#      Test procedure component 


set CurrentScript "$env(HOME)/AgataTests/undefined.tcl"
set CurrentVariable 0

set %%Waiting 0

set %%Digitizer 0
set %%Fpga $Fpga


proc Hardware_Tests {} {
   global env Digitizer FpgaList Fpga %%Fpga %%Digitizer
   global inputcolour outputcolour insertcolour bkgcolour

set rows 16
set cols 80

 set %%Fpga $Fpga
 set %%Digitizer $Digitizer

  :frame [self frame-name].tests :label "[self frame-title]: Hardware Tests" :icon-name "[self icon-title]"

   :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10
     :choice-stack TFPGA  :label "FPGA"  :exclusive T :strings $FpgaList \
         :value [lsearch $FpgaList [set %%Fpga]] :command click-TFpga
     :hskip 250
     :button TCLEAR   :label "Empty Log Window" :command ":modify TESTLOG :empty T"

   :panel P1 :layout V :padx 5 :pady 5
    :galley TESTLOG :rows $rows :cols $cols :font fixed

   :panel P2 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10
     :text CSCRIPT :label "Current Script" :w 64 :deaf T
     :next-row T
     :button SSCRIPT :label "Select Script" :command SelectScript
     :button PSCRIPT :label "Previous"      :command PreviousScript
     :button NSCRIPT :label "Next"          :command NextScript
     :hskip 100
     :button RSCRIPT :label "Run Script" :w 32 :command RunScript

     :next-row T
     :message WAITING :label "" :bold F :w 32
     :message WAITMSG :label "" :bold F :w 32
     :button  WAITCONT :label "Continue" :command ContinueScript
     :button  WAITABRT :label "Abort"    :command AbortScript

     :next-row T
     :button SHOW :label "Show Variables"
     :text SET1 :label "Set Variable"
     :text SET2 :label "="


   :panel LAST :layout V :padx 5 :pady 5
    :layout H; :col-gap 20

   open-frame

  :galley_size TESTLOG $rows $cols

    :modify TESTLOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify TESTLOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify TESTLOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"

    :show T

  set-footer [self frame-name].tests "Currently selected digitizer=$%%Digitizer"

}

#proc click-configure {W} {:galley_resize TESTLOG}

proc InsertLog {s} {:modify TESTLOG :usingtag taginsert :strings $s\n}

proc SelectScript {args} {
    global env CurrentScript

    	option add *Canvas.background  #c0c020
	set types {
	{ {Tcl Script Files} {*.tcl} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "Script file selection"  \
		  -initialdir [file dirname $CurrentScript] -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

      set CurrentScript $f
      :modify CSCRIPT :value $CurrentScript
}

proc PreviousScript {args} {
    global env CurrentScript

    set f1 [first [split [file tail $CurrentScript] .]]
    set f2 [first [split $f1 0123456789]]
    set f3 [string range $f1 [string length $f2] end]

    if {[string length $f3] == 0} {midas-warning "Current Script filename ($CurrentScript) does not have a suitable format for this operation"; return}

    incr f3 -1

    set NewScript [file join [file dirname $CurrentScript] [set f2][set f3][file extension $CurrentScript]]

    file exists $NewScript
    if {[file exists $NewScript] == 0} {midas-warning "New Script filename ($NewScript) does not exist"; return}

    set CurrentScript $NewScript
    :modify CSCRIPT :value $CurrentScript
}

proc NextScript {args} {
    global env CurrentScript

    set f1 [first [split [file tail $CurrentScript] .]]
    set f2 [first [split $f1 0123456789]]
    set f3 [string range $f1 [string length $f2] end]

    if {[string length $f3] == 0} {midas-warning "Current Script filename ($CurrentScript) does not have a suitable format for this operation"; return}

    incr f3

    set NewScript [file join [file dirname $CurrentScript] [set f2][set f3][file extension $CurrentScript]]

    file exists $NewScript
    if {[file exists $NewScript] == 0} {midas-warning "New Script filename ($NewScript) does not exist"; return}

    set CurrentScript $NewScript
    :modify CSCRIPT :value $CurrentScript
}

proc RunScript {args} {
    global env CurrentScript

    :modify RSCRIPT :inactive T
    set rc [catch {uplevel #0 source $CurrentScript} m]
    :modify RSCRIPT :inactive F

    if {$rc != 0} {midas-warning "Script returned an error: $m"}

}

proc WaitForUser {msg} {
    global %%Waiting

    :modify WAITING :label "Script waiting for user" :bold T
    [:path WAITING].msg configure -fg red
    :modify WAITMSG :label $msg :bold T
    [:path WAITMSG].msg configure -fg red

    set %%Waiting 1

    vwait %%Waiting

    :modify WAITING :label "There is no current wait pending"
    [:path WAITING].msg configure -fg white
    :modify WAITMSG :label "Wait complete"
    [:path WAITMSG].msg configure -fg white
}

proc ContinueScript {args} {
    global %%Waiting

     set %%Waiting -1
}

proc AbortScript {args} {
    global %%Waiting

     set %%Waiting -2
}

proc click-show {args} {

    foreach var [info globals %%*] {
        global $var
        InsertLog "$var = [set $var]"
    }
}

proc click-set1 {args} {
    global CurrentVariable 

    set CurrentVariable  $args
}

proc click-set2 {args} {
    global CurrentVariable 

    global $CurrentVariable
    set $CurrentVariable  $args

    InsertLog "$CurrentVariable = [set $CurrentVariable]"
}


proc SelectDigitizer {n}  {global %%Digitizer; set %%Digitizer $n}

proc SelectFpga {n} {
    global  %%Digitizer  %%Fpga  FpgaList  FpgaIDs

   if {[lsearch $FpgaList $n] == -1 || [lindex $FpgaIDs [lsearch $FpgaList $n]] == -1} {
       InsertLog "$n is a bad Fpga"
       return -1
   } else {
       set %%Fpga $n
       :modify TFPGA :value [lsearch $FpgaList [set %%Fpga]]
   }
   return [set %%Fpga]
}

proc click-TFpga {w vl i} {SelectFpga $i}

proc Set_Register {Addr Vl {Space A16}}  {
    global  %%Digitizer  %%Fpga  FpgaList  FpgaIDs
    global errorCode errorInfo

        set fpga [lindex $FpgaIDs [lsearch $FpgaList [set %%Fpga]]]

        set rc [catch {AGATADigitizerAccessClient__Set [set %%Digitizer] $fpga $Addr $Space $Vl} m]
        if {$rc != 0} {
            InsertLog  "Set returned with an error\n$m\n$errorCode\n$errorInfo"
            return -1
        } else {
            return $m
        }
}

proc Get_Register {Addr {Space A16}}  {
    global  %%Digitizer  %%Fpga  FpgaList  FpgaIDs
    global errorCode errorInfo

        set fpga [lindex $FpgaIDs [lsearch $FpgaList [set %%Fpga]]]

        set rc [catch {AGATADigitizerAccessClient__Get [set %%Digitizer] $fpga $Addr $Space} m]
        if {$rc != 0} {
            InsertLog  "Get returned with an error\n$m\n$errorCode\n$errorInfo"
            return -1
        } else {
            return $m
        }
}
