#    This is a simple terminal emulation program (like Hyperterminal or Teraterm)

set frame  terminal
set title "Terminal"
set icon  terminal

set started 0

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set PortList {null}    ;#  silly default

# more variables for storing Log data 
set Logcount 0 
set Logstatus { "0000" "0000" "0000" "0000" "0000" "0000" }
set Logdata   { "0000" "0000" "0000" "0000" "0000" "0000" }
set Logerror  { "0000" "0000" "0000" "0000" "0000" "0000" }

set Logstring ""
# end of Log variables

   switch $env(platform) {
     windows  {
         set PortList {COM1 COM2 COM3 COM4 COM5 COM6}
     }
     unix {
        switch $env(os) {
           SunOS    {
               set PortList {/dev/ttya /dev/ttyb}
           }
           Linux  {
               set PortList {/dev/ttyS0 /dev/ttyS1}
           }
           default  {puts stderr "$env(os) not supported"}
        }
     }
     default {puts stderr "$env(platform) not supported"}
   }

set Port [lindex $PortList 0]

set BaudList {9600 600 1200 2400 4800 9600 14400 19200 38400 57600 115200}
set Baud [lindex $BaudList 0]

set DataList {8 7 6 5}
set Data [lindex $DataList 0]

set ParityList {none odd even mark space}
set Parity [lindex $ParityList 0]

set StopList {1 2}
set Stop [lindex $StopList 0]

set FlowList {none rtscts xonxoff}
set Flow [lindex $FlowList 0]

set Cmd ""

set channelId 0

proc make-frame {} {
   global env PortList BaudList DataList ParityList StopList FlowList

set rows 24
set cols 80

  :frame [self frame-name] :label "[self frame-title] for Patrick : Rocket I/O V2P log format" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10
    :choice-stack PORT   :label "Port" :strings $PortList
    :choice-stack BAUD   :label "Baud rate" :strings $BaudList
    :choice-stack DATA   :label "Data bits" :strings $DataList 
    :choice-stack PARITY :label "Parity" :strings $ParityList
    :choice-stack STOP   :label "Stop bits" :strings $StopList
    :choice-stack FLOW   :label "Flow" :strings $FlowList

    :next-row T
    :button FINISH     :label Finish     :command do_finish
    :hskip 100
    :button CONNECT    :label Connect    :w 12 :command do_connect
    :button DISCONNECT :label Disconnect :w 12 :command do_disconnect
    :hskip 100
    :button CLEAR   :label "Clear log window" :command ":modify LOG :empty T"

  :panel P1 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font fixed
     configuretags

  :panel P3 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10
    :text COMMAND :label "" :w $cols 

   open-frame

  :galley_size LOG $rows $cols

    focus [:path COMMAND].entry

   :show T
}

proc click-configure {W} {:galley_resize LOG}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n}

proc start args {
   global started

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
}

proc click arg {
   global env PortList BaudList DataList ParityList StopList FlowList
   global Port Baud Data Parity Stop Flow
   global Cmd

   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer
   switch -glob $w  {

      PORT  {set Port [lindex $PortList $v]}
      BAUD  {set Baud [lindex $BaudList $v]}
      DATA  {set Data [lindex $DataList $v]}
      PARITY  {set Parity [lindex $ParityList $v]}
      STOP  {set Stop [lindex $StopList $v]}
      FLOW  {set Flow [lindex $FlowList $v]}

      COMMAND {set Cmd $v; do_command}

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc do_finish {} {
    global channelId 

   if {$channelId != 0} {do_disconnect}
   DeleteTask
}

proc do_command {} {
    global channelId Cmd

#   :modify LOG :usingtag taginput :strings "$Cmd\n"

   if {$channelId == 0} {insert "Not connected"; return}

   puts $channelId "$Cmd"
   :modify COMMAND :value ""
}

proc do_connect {} {
   global channelId Port
   global Port Baud Data Parity Stop Flow

   if {$channelId != 0} {
       insert "Already connected: Disconnect first"
   } else {
       set channelId [open $Port RDWR]
       switch $Parity {
           none   {set parity n}
           odd    {set parity o}
           even   {set parity e}
           mark   {set parity m}
           space  {set parity s}
       }
    
       fconfigure $channelId -mode $Baud,$parity,$Data,$Stop -handshake $Flow
       fconfigure $channelId -blocking no -buffering none -translation binary
       fileevent $channelId readable GetData
       insert "Connected to channel $channelId"
   }
}

proc do_disconnect {} {
   global channelId Port

   if {$channelId == 0} {
       insert "Already disconnected"
   } else {
       close $channelId
       insert "Channel $channelId disconnected"
       set channelId 0
   }
}
# DC:22:59:54 :0000:0000:0000:0000:0000:0000:2E9D:FC00:FC00:FC00:FC00:FC00:FC00:DA01:DA01:DA01:DA01:DA01:DA01
 #   set data " :0001:0002:0003:0004:0005:0006:2E9D:FC01:FC02:FC03:FC04:FC05:FC06:DA01:DA01:DA01:DA01:DA01:DA01"

proc GetData {args} {
    global channelId  Logstring    
	set valstr 0
	after 1000
    	set data [read $channelId 132]
	set bitlength [string length $data] 
	set bitlength [ expr $bitlength - 1 ]
	set val [string range $data $bitlength $bitlength ]
	scan $val %c valstr
	if { $valstr == 10 } {
		set Logstring $data
  		for {set i 0} {$i < 6 } {incr i} {
			set strt [ expr ( $i * 5 ) + 13  ]
			set fin  [ expr $strt + 3 ]  
     	   		set c [string range $Logstring $strt $fin]
    	   		lappend Logerror $c

			set strt [ expr ( $i * 5 ) + 78  ]
			set fin  [ expr $strt + 3 ]  
     	   		set c [string range $Logstring $strt $fin]
    	   		lappend Logstatus $c

			set strt [ expr ( $i * 5 ) + 48  ]
			set fin  [ expr $strt + 3 ]  
     	   		set c [string range $Logstring $strt $fin]
    	   		lappend Logdata $c

   		}
		set strt  43  
		set fin  [ expr $strt + 3 ]  
    		set Logcount [string range $Logstring $strt $fin]
   		insert "Log number  :$Logcount"
		insert "Channel     :  1    2    3    4    5    6"
   		insert "Data        :$Logdata"
   		insert "Error       :$Logerror"
	   	insert "Status      :$Logstatus"
		insert " "
 
	}
}
#   		if {[string length $line] == 0} {insert "null line received"; do_disconnect; after 1000; do_connect}
#   		:modify LOG :usingtag tagoutput :strings "$line"
